/*
 * CS 4481 Compilers
 * Project 2 - scanner part 2
 *
 * This file contains the method runTests(). You will uncomment tests
 * as you develop your scanner code
 */
package scanner;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;

/**
 *
 * @author edwajohn
 */
public class Main {

  public static void main(String[] args)
          throws FileNotFoundException, IOException {
    if (args.length > 0) {
      scanInputFile(args[0], args[1]);
    } else {
      int failures = runTests();
      System.exit(failures);
    }
  }

  public static int runTests() throws FileNotFoundException, IOException {
    Tests tests = new Tests();

    // TODO: uncomment tests as you develop code

    String tableFile = "data/pasca.table";
    TableParser tableParser = new TableParser(tableFile);
    Scanner scanner = new Scanner(tableParser);

    {
      //------------------------------------------------------------
      // Test input test1.pas
      //------------------------------------------------------------

      ScanStream ss = getDataStream("data/test1.pas");

      // testNext skips tokens of type ignore.
      tests.test(Token.testNext("identifier", "begin", scanner, ss));
      tests.test(Token.testNext("identifier", "program", scanner, ss));
      tests.test(Token.testNext("identifier", "a1", scanner, ss));
      tests.test(Token.testNext("delimiter", "{", scanner, ss));
      tests.test(Token.testNext("identifier", "var0", scanner, ss));
      tests.test(Token.testNext("identifier", "is", scanner, ss));
      tests.test(Token.testNext("integer", "8", scanner, ss));
      tests.test(Token.testNext("delimiter", ";", scanner, ss));
      tests.test(Token.testNext("identifier", "var1", scanner, ss));
      tests.test(Token.testNext("identifier", "var2", scanner, ss));
      tests.test(Token.testNext("identifier", "variable", scanner, ss));
      tests.test(Token.testNext("identifier", "var1able", scanner, ss));
      tests.test(Token.testNext("delimiter", "}", scanner, ss));
      tests.test(Token.testNext("integer", "123", scanner, ss));
      tests.test(Token.testNext("integer", "1203", scanner, ss));
      tests.test(Token.testNext("string", "' a crazy string abc(){}*''4013'",
                                scanner, ss));
      tests.test(Token.testNext("identifier", "end", scanner, ss));
      tests.test(Token.testNext("delimiter", ".", scanner, ss));
    }

    {
      //------------------------------------------------------------
      // Test input test2.pas
      //------------------------------------------------------------

      ScanStream ss = getDataStream("data/test2.pas");

      tests.test(Token.testNext("string", "'alks dfl;alsk k''jk{}'",
                                scanner, ss));
    }

    {
      //------------------------------------------------------------
      // Test input err1.pas
      //------------------------------------------------------------

      ScanStream ss = getDataStream("data/err1.pas");
      Token t = scanner.parseToken(ss);
      tests.test(t.test("ignore", "(* a comment *)"));
      t = scanner.parseToken(ss);
      tests.test(t.test("ignore", " "));
      t = scanner.parseToken(ss);
      tests.test(t == null);
    }

    {
      //------------------------------------------------------------
      // Test input err2.pas
      //------------------------------------------------------------

      ScanStream ss = getDataStream("data/err2.pas");
      Token t = scanner.parseToken(ss);
      tests.test(t == null);
    }

    {
      //------------------------------------------------------------
      // Test input err3.pas
      //------------------------------------------------------------

      ScanStream ss = getDataStream("data/err3.pas");
      Token t = scanner.parseToken(ss);
      tests.test(t == null);
    }

    //------------------------------------------------------------
    // Output number of tests that succeeded
    //------------------------------------------------------------
    System.out.println(tests.getSuccesses() + "/" + tests.getN()
            + " tests succeeded");
    return tests.getFailures();
  }

  private static ScanStream getDataStream(String testFile) throws IOException {
    File file = new File(testFile);
    FileInputStream fis = new FileInputStream(file);
    byte[] data = new byte[(int) file.length()];
    fis.read(data);
    fis.close();
    ScanStream ss = new ScanStream(data);
    return ss;
  }

  public static void scanInputFile(String tableFile, String testFile)
          throws FileNotFoundException, IOException {
    TableParser tableParser = new TableParser(tableFile);

    File file = new File(testFile);
    FileInputStream fis = new FileInputStream(file);
    byte[] data = new byte[(int) file.length()];
    fis.read(data);
    fis.close();

    PrintWriter writer = new PrintWriter(testFile + ".out", "UTF-8");

    Scanner scanner = new Scanner(tableParser);
    ScanStream ss = new ScanStream(data);
    Token t;
    do {
      t = scanner.parseToken(ss);
      if (t == null) {
        System.out.println("Lexical error: " + ss.next());
        ss.rollback();
        writer.println("Lexical error: " + ss.next());
      } else if (!t.getType().equals("ignore")) {
        System.out.println(t.toString());
        writer.println(t.toString());
      }
    } while (t != null && !ss.eof());

    writer.close();
  }


}
